@@ -49,7 +49,7 @@ public:
     JobPrivate()
         : m_parentJob(nullptr), m_extraFlags(0),
           m_uiDelegateExtension(KIO::defaultJobUiDelegateExtension()),
-          m_privilegeExecutionEnabled(false), m_confirmationAsked(false)
+          m_privilegeExecutionEnabled(false)
     {
     }
 
@@ -89,11 +89,10 @@ public:
     Job *q_ptr;
     // For privilege operation
     bool m_privilegeExecutionEnabled;
-    bool m_confirmationAsked;
     QString m_caption, m_message;
     FileOperationType m_operationType;
 
-    PrivilegeOperationStatus tryAskPrivilegeOpConfirmation();
+    QByteArray privilegeOperationData();
     void slotSpeed(KJob *job, unsigned long speed);
 
     static void emitMoving(KIO::Job *, const QUrl &src, const QUrl &dest);
@@ -263,6 +262,8 @@ public:
             case CMD_SYMLINK:
                 opType = Symlink;
                 break;
+            default:
+                return job;
             }
             job->d_func()->m_operationType = opType;
         }
@@ -314,7 +315,7 @@ public:
      * work on this job.
      * @param slave the slave that works on the job
      */
-    void start(KIO::Slave *slave) Q_DECL_OVERRIDE;
+    void start(KIO::Slave *slave) override;
     /**
      * @internal
      * Called when the ioslave needs the data to send the server. This slot
@@ -324,9 +325,6 @@ public:
     virtual void slotDataReqFromDevice();
     void slotIODeviceClosed();
     void slotIODeviceClosedBeforeStart();
-
-    void slotErrorPage();
-    void slotCanResume(KIO::filesize_t offset);
     void slotPostRedirection();
     void slotNeedSubUrlData();
     void slotSubUrlData(KIO::Job *, const QByteArray &);
