@@ -92,6 +92,7 @@ public:
             QStandardPaths::setTestModeEnabled(true);
         }
         pendingListEntries.reserve(KIO_MAX_ENTRIES_PER_BATCH);
+        appConnection.setReadMode(Connection::ReadMode::Polled);
     }
     ~SlaveBasePrivate()
     {
@@ -755,7 +756,7 @@ void SlaveBase::listEntries(const UDSEntryList &list)
     QByteArray data;
     QDataStream stream(&data, QIODevice::WriteOnly);
 
-    foreach (const UDSEntry &entry, list) {
+    for (const UDSEntry &entry : list) {
         stream << entry;
     }
 
@@ -1042,7 +1043,7 @@ int SlaveBase::readData(QByteArray &buffer)
 void SlaveBase::setTimeoutSpecialCommand(int timeout, const QByteArray &data)
 {
     if (timeout > 0) {
-        d->nextTimeoutMsecs = timeout*1000; // from seconds to miliseconds
+        d->nextTimeoutMsecs = timeout*1000; // from seconds to milliseconds
         d->nextTimeout.start();
     } else if (timeout == 0) {
         d->nextTimeoutMsecs = 1000;  // Immediate timeout
