@@ -22,10 +22,6 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef QT_NO_CAST_FROM_ASCII
-#define QT_NO_CAST_FROM_ASCII
-#endif
-
 #include "file.h"
 
 #include <config-kioslave-file.h>
@@ -48,16 +44,16 @@
 #endif
 
 
-#include <QtCore/QByteRef>
-#include <QtCore/QDate>
-#include <QtCore/QVarLengthArray>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QRegExp>
-#include <QtCore/QFile>
+#include <QByteRef>
+#include <QDate>
+#include <QVarLengthArray>
+#include <QCoreApplication>
+#include <QRegExp>
+#include <QFile>
 #include <qtemporaryfile.h>
 #ifdef Q_OS_WIN
-#include <QtCore/QDir>
-#include <QtCore/QFileInfo>
+#include <QDir>
+#include <QFileInfo>
 #endif
 
 #include <QDebug>
@@ -69,6 +65,10 @@
 #include <QStandardPaths>
 #include <QDataStream>
 
+#if HAVE_STATX
+#include <sys/stat.h>
+#endif
+
 #if HAVE_VOLMGT
 #include <volmgt.h>
 #include <sys/mnttab.h>
@@ -188,7 +188,7 @@ int FileProtocol::setACL(const char *path, mode_t perm, bool directoryDefault)
             // the minimal (UNIX permission bits) part
             acl = acl_from_mode(perm);
         }
-        acl = acl_from_text(ACLString.toLatin1());
+        acl = acl_from_text(ACLString.toLatin1().constData());
         if (acl_valid(acl) == 0) {     // let's be safe
             ret = acl_set_file(path, ACL_TYPE_ACCESS, acl);
             // qDebug() << "Set ACL on:" << path << "to:" << aclToText(acl);
@@ -204,7 +204,7 @@ int FileProtocol::setACL(const char *path, mode_t perm, bool directoryDefault)
             // user told us to delete the default ACL, do so
             ret += acl_delete_def_file(path);
         } else {
-            acl_t acl = acl_from_text(defaultACLString.toLatin1());
+            acl_t acl = acl_from_text(defaultACLString.toLatin1().constData());
             if (acl_valid(acl) == 0) {     // let's be safe
                 ret += acl_set_file(path, ACL_TYPE_DEFAULT, acl);
                 // qDebug() << "Set Default ACL on:" << path << "to:" << aclToText(acl);
@@ -267,7 +267,7 @@ void FileProtocol::setModificationTime(const QUrl &url, const QDateTime &mtime)
     if (QT_LSTAT(QFile::encodeName(path).constData(), &statbuf) == 0) {
         struct utimbuf utbuf;
         utbuf.actime = statbuf.st_atime; // access time, unchanged
-        utbuf.modtime = mtime.toTime_t(); // modification time
+        utbuf.modtime = mtime.toSecsSinceEpoch(); // modification time
         if (::utime(QFile::encodeName(path).constData(), &utbuf) != 0) {
             if (auto err = execWithElevatedPrivilege(UTIME, {path, qint64(utbuf.actime), qint64(utbuf.modtime)}, errno)) {
                 if (!err.wasCanceled()) {
@@ -329,13 +329,29 @@ void FileProtocol::mkdir(const QUrl &url, int permissions)
     return;
 }
 
+void FileProtocol::redirect(const QUrl &url)
+{
+    QUrl redir(url);
+    redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
+
+    // if we would redirect into the Windows world, let's also check for the
+    // DavWWWRoot "token" which in the Windows world tells win explorer to access
+    // a webdav url
+    // https://www.webdavsystem.com/server/access/windows
+    if ((redir.scheme() == QLatin1String("smb")) &&
+        redir.path().startsWith(QLatin1String("/DavWWWRoot/"))) {
+        redir.setPath(redir.path().mid(11));  // remove /DavWWWRoot
+        redir.setScheme(QStringLiteral("webdav"));
+    }
+
+    redirection(redir);
+    finished();
+}
+
 void FileProtocol::get(const QUrl &url)
 {
     if (!url.isLocalFile()) {
-        QUrl redir(url);
-        redir.setScheme(config()->readEntry("DefaultRemoteProtocol", "smb"));
-        redirection(redir);
-        finished();
+        redirect(url);
         return;
     }
 
@@ -381,7 +397,7 @@ void FileProtocol::get(const QUrl &url)
     // need, so for local files, better use mimeTypeForFile.
     QMimeDatabase db;
     QMimeType mt = db.mimeTypeForFile(url.toLocalFile());
-    emit mimeType(mt.name());
+    mimeType(mt.name());
     // Emit total size AFTER mimetype
     totalSize(buff.st_size);
 
@@ -479,13 +495,13 @@ void FileProtocol::open(const QUrl &url, QIODevice::OpenMode mode)
     if (mode & QIODevice::ReadOnly) {
         QMimeDatabase db;
         QMimeType mt = db.mimeTypeForFile(url.toLocalFile());
-        emit mimeType(mt.name());
+        mimeType(mt.name());
     }
 
     totalSize(buff.st_size);
     position(0);
 
-    emit opened();
+    opened();
 }
 
 void FileProtocol::read(KIO::filesize_t bytes)
@@ -493,7 +509,6 @@ void FileProtocol::read(KIO::filesize_t bytes)
     // qDebug() << "File::open -- read";
     Q_ASSERT(mFile && mFile->isOpen());
 
-    QVarLengthArray<char> buffer(bytes);
     while (true) {
         QByteArray res = mFile->read(bytes);
 
@@ -603,7 +618,6 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
 
     int result;
     QString dest;
-    QByteArray _dest;
     QFile f;
 
     // Loop until we got 0 (end of data)
@@ -774,13 +788,13 @@ void FileProtocol::put(const QUrl &url, int _mode, KIO::JobFlags _flags)
                 utbuf[0].tv_sec = dest_statbuf.st_atime; // access time, unchanged  ## TODO preserve msec
                 utbuf[0].tv_usec = 0;
                 // modification time
-                utbuf[1].tv_sec = dt.toTime_t();
+                utbuf[1].tv_sec = dt.toSecsSinceEpoch();
                 utbuf[1].tv_usec = dt.time().msec() * 1000;
                 utimes(QFile::encodeName(dest_orig).constData(), utbuf);
 #else
                 struct utimbuf utbuf;
                 utbuf.actime = dest_statbuf.st_atime;
-                utbuf.modtime = dt.toTime_t();
+                utbuf.modtime = dt.toSecsSinceEpoch();
                 if (utime(QFile::encodeName(dest_orig).constData(), &utbuf) != 0) {
                     tryChangeFileAttr(UTIME, {dest_orig, qint64(utbuf.actime), qint64(utbuf.modtime)}, errno);
                 }
@@ -799,16 +813,16 @@ QString FileProtocol::getUserName(KUserId uid) const
     if (Q_UNLIKELY(!uid.isValid())) {
         return QString();
     }
-    if (!mUsercache.contains(uid)) {
+    auto it = mUsercache.find(uid);
+    if (it == mUsercache.end()) {
         KUser user(uid);
         QString name = user.loginName();
         if (name.isEmpty()) {
             name = uid.toString();
         }
-        mUsercache.insert(uid, name);
-        return name;
+        it = mUsercache.insert(uid, name);
     }
-    return mUsercache[uid];
+    return *it;
 }
 
 QString FileProtocol::getGroupName(KGroupId gid) const
@@ -816,46 +830,123 @@ QString FileProtocol::getGroupName(KGroupId gid) const
     if (Q_UNLIKELY(!gid.isValid())) {
         return QString();
     }
-    if (!mGroupcache.contains(gid)) {
+    auto it = mGroupcache.find(gid);
+    if (it == mGroupcache.end()) {
         KUserGroup group(gid);
         QString name = group.name();
         if (name.isEmpty()) {
             name = gid.toString();
         }
-        mGroupcache.insert(gid, name);
-        return name;
+        it = mGroupcache.insert(gid, name);
     }
-    return mGroupcache[gid];
+    return *it;
 }
 
+#if HAVE_STATX
+// statx syscall is available
+inline int LSTAT(const char* path, struct statx * buff) {
+    return statx(AT_FDCWD, path, AT_SYMLINK_NOFOLLOW, STATX_BASIC_STATS | STATX_BTIME, buff);
+}
+inline int STAT(const char* path, struct statx * buff) {
+    return statx(AT_FDCWD, path, AT_STATX_SYNC_AS_STAT, STATX_BASIC_STATS | STATX_BTIME, buff);
+}
+inline static uint16_t stat_mode(struct statx &buf) { return buf.stx_mode; }
+inline static uint32_t stat_dev(struct statx &buf) { return buf.stx_dev_major; }
+inline static uint64_t stat_ino(struct statx &buf) { return buf.stx_ino; }
+inline static uint64_t stat_size(struct statx &buf) { return buf.stx_size; }
+inline static uint32_t stat_uid(struct statx &buf) { return buf.stx_uid; }
+inline static uint32_t stat_gid(struct statx &buf) { return buf.stx_gid; }
+inline static int64_t stat_atime(struct statx &buf) { return buf.stx_atime.tv_sec; }
+inline static int64_t stat_mtime(struct statx &buf) { return buf.stx_mtime.tv_sec; }
+#else
+// regular stat struct
+inline int LSTAT(const char* path, QT_STATBUF * buff) {
+    return QT_LSTAT(path, buff);
+}
+inline int STAT(const char* path, QT_STATBUF * buff) {
+    return QT_STAT(path, buff);
+}
+inline static mode_t stat_mode(QT_STATBUF &buf) { return buf.st_mode; }
+inline static dev_t stat_dev(QT_STATBUF &buf) { return buf.st_dev; }
+inline static ino_t stat_ino(QT_STATBUF &buf) { return buf.st_ino; }
+inline static off_t stat_size(QT_STATBUF &buf) { return buf.st_size; }
+#ifndef Q_OS_WIN
+inline static uid_t stat_uid(QT_STATBUF &buf) { return buf.st_uid; }
+inline static gid_t stat_gid(QT_STATBUF &buf) { return buf.st_gid; }
+#endif
+inline static time_t stat_atime(QT_STATBUF &buf) { return buf.st_atime; }
+inline static time_t stat_mtime(QT_STATBUF &buf) { return buf.st_mtime; }
+#endif
+
 bool FileProtocol::createUDSEntry(const QString &filename, const QByteArray &path, UDSEntry &entry,
                                   short int details)
 {
     assert(entry.count() == 0); // by contract :-)
-    entry.reserve(8);
-
-    entry.insert(KIO::UDSEntry::UDS_NAME, filename);
+    switch (details) {
+    case 0:
+        // filename, access, type, size, linkdest
+        entry.reserve(5);
+        break;
+    case 1:
+        // uid, gid, atime, mtime, btime
+        entry.reserve(10);
+        break;
+    case 2:
+        // acl data
+        entry.reserve(13);
+        break;
+    default: // case details > 2
+        // dev, inode
+        entry.reserve(15);
+        break;
+    }
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, filename);
 
     mode_t type;
     mode_t access;
-    QT_STATBUF buff;
+    bool isBrokenSymLink = false;
+    signed long long size = 0LL;
+#if HAVE_POSIX_ACL
+    QByteArray targetPath = path;
+#endif
 
-    if (QT_LSTAT(path.data(), &buff) == 0)  {
+#if HAVE_STATX
+    // statx syscall is available
+    struct statx buff;
+#else
+    QT_STATBUF buff;
+#endif
+
+    if (LSTAT(path.data(), &buff) == 0)  {
 
         if (details > 2) {
-            entry.insert(KIO::UDSEntry::UDS_DEVICE_ID, buff.st_dev);
-            entry.insert(KIO::UDSEntry::UDS_INODE, buff.st_ino);
+            entry.fastInsert(KIO::UDSEntry::UDS_DEVICE_ID, stat_dev(buff));
+            entry.fastInsert(KIO::UDSEntry::UDS_INODE, stat_ino(buff));
         }
 
-        if ((buff.st_mode & QT_STAT_MASK) == QT_STAT_LNK) {
+        if ((stat_mode(buff) & QT_STAT_MASK) == QT_STAT_LNK) {
 
 #ifdef Q_OS_WIN
             const QString linkTarget = QFile::symLinkTarget(QFile::decodeName(path));
 #else
             // Use readlink on Unix because symLinkTarget turns relative targets into absolute (#352927)
-            const off_t lowerLimit = 1;
-            const off_t upperLimit = 1024;
-            size_t bufferSize = qBound(lowerLimit, buff.st_size, upperLimit);
+            #if HAVE_STATX
+                size_t lowerBound = 256;
+                size_t higherBound = 1024;
+                uint64_t s = stat_size(buff);
+                if (s > SIZE_MAX) {
+                    qCWarning(KIO_FILE) << "file size bigger than SIZE_MAX, too big for readlink use!" << path;
+                    return false;
+                }
+                size_t size = static_cast<size_t>(s);
+                using SizeType = size_t;
+            #else
+                off_t lowerBound = 256;
+                off_t higherBound = 1024;
+                off_t size = stat_size(buff);
+                using SizeType = off_t;
+            #endif
+            SizeType bufferSize = qBound(lowerBound, size +1, higherBound);
             QByteArray linkTargetBuffer;
             linkTargetBuffer.resize(bufferSize);
             while (true) {
@@ -863,7 +954,9 @@ bool FileProtocol::createUDSEntry(const QString &filename, const QByteArray &pat
                 if (n < 0 && errno != ERANGE) {
                     qCWarning(KIO_FILE) << "readlink failed!" << path;
                     return false;
-                } else if (n > 0 && static_cast<size_t>(n) != bufferSize) {
+                } else if (n > 0 && static_cast<SizeType>(n) != bufferSize) {
+                    // the buffer was not filled in the last iteration
+                    // we are finished reading, break the loop
                     linkTargetBuffer.truncate(n);
                     break;
                 }
@@ -872,19 +965,18 @@ bool FileProtocol::createUDSEntry(const QString &filename, const QByteArray &pat
             }
             const QString linkTarget = QFile::decodeName(linkTargetBuffer);
 #endif
-            entry.insert(KIO::UDSEntry::UDS_LINK_DEST, linkTarget);
+            entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, linkTarget);
 
             // A symlink -> follow it only if details>1
-            if (details > 1 && QT_STAT(path.constData(), &buff) == -1) {
-                // It is a link pointing to nowhere
-                type = S_IFMT - 1;
-                access = S_IRWXU | S_IRWXG | S_IRWXO;
-
-                entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, type);
-                entry.insert(KIO::UDSEntry::UDS_ACCESS, access);
-                entry.insert(KIO::UDSEntry::UDS_SIZE, 0LL);
-                goto notype;
-
+            if (details > 1) {
+                if (STAT(path.constData(), &buff) == -1) {
+                    isBrokenSymLink = true;
+                } else {
+#if HAVE_POSIX_ACL
+                    // valid symlink, will get the ACLs of the destination
+                    targetPath = linkTargetBuffer;
+#endif
+                }
             }
         }
     } else {
@@ -892,33 +984,40 @@ bool FileProtocol::createUDSEntry(const QString &filename, const QByteArray &pat
         return false;
     }
 
-    type = buff.st_mode & S_IFMT; // extract file type
-    access = buff.st_mode & 07777; // extract permissions
+    if (isBrokenSymLink) {
+        // It is a link pointing to nowhere
+        type = S_IFMT - 1;
+        access = S_IRWXU | S_IRWXG | S_IRWXO;
+        size = 0LL;
+    } else {
+        type = stat_mode(buff) & S_IFMT; // extract file type
+        access = stat_mode(buff) & 07777; // extract permissions
+        size = stat_size(buff);
+    }
 
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, type);
-    entry.insert(KIO::UDSEntry::UDS_ACCESS, access);
-
-    entry.insert(KIO::UDSEntry::UDS_SIZE, buff.st_size);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, type);
+    entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, access);
+    entry.fastInsert(KIO::UDSEntry::UDS_SIZE, size);
 
 #if HAVE_POSIX_ACL
     if (details > 1) {
         /* Append an atom indicating whether the file has extended acl information
          * and if withACL is specified also one with the acl itself. If it's a directory
          * and it has a default ACL, also append that. */
-        appendACLAtoms(path, entry, type);
+        appendACLAtoms(targetPath, entry, type);
     }
 #endif
 
-notype:
     if (details > 0) {
-        entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, buff.st_mtime);
+        entry.fastInsert(KIO::UDSEntry::UDS_MODIFICATION_TIME, stat_mtime(buff));
+        entry.fastInsert(KIO::UDSEntry::UDS_ACCESS_TIME, stat_atime(buff));
 #ifndef Q_OS_WIN
-        entry.insert(KIO::UDSEntry::UDS_USER, getUserName(KUserId(buff.st_uid)));
-        entry.insert(KIO::UDSEntry::UDS_GROUP, getGroupName(KGroupId(buff.st_gid)));
+        entry.fastInsert(KIO::UDSEntry::UDS_USER, getUserName(KUserId(stat_uid(buff))));
+        entry.fastInsert(KIO::UDSEntry::UDS_GROUP, getGroupName(KGroupId(stat_gid(buff))));
 #else
 #pragma message("TODO: st_uid and st_gid are always zero, use GetSecurityInfo to find the owner")
 #endif
-        entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME, buff.st_atime);
+
 #ifdef st_birthtime
         /* For example FreeBSD's and NetBSD's stat contains a field for
          * the inode birth time: st_birthtime
@@ -926,23 +1025,21 @@ notype:
          * Instead of setting a bogus fallback like st_mtime, only use
          * it if it is greater than 0. */
         if (buff.st_birthtime > 0) {
-            entry.insert(KIO::UDSEntry::UDS_CREATION_TIME, buff.st_birthtime);
+            entry.fastInsert(KIO::UDSEntry::UDS_CREATION_TIME, buff.st_birthtime);
         }
-#endif
-#ifdef __st_birthtime
+#elif defined __st_birthtime
         /* As above, but OpenBSD calls it slightly differently. */
         if (buff.__st_birthtime > 0) {
-            entry.insert(KIO::UDSEntry::UDS_CREATION_TIME, buff.__st_birthtime);
+            entry.fastInsert(KIO::UDSEntry::UDS_CREATION_TIME, buff.__st_birthtime);
+        }
+#elif HAVE_STATX
+        /* And linux version using statx syscall */
+        if (buff.stx_mask & STATX_BTIME) {
+            entry.fastInsert(KIO::UDSEntry::UDS_CREATION_TIME, buff.stx_btime.tv_sec);
         }
 #endif
     }
 
-    // Note: buff.st_ctime isn't the creation time !
-    // We made that mistake for KDE 2.0, but it's in fact the
-    // "file status" change time, which we don't care about.
-    // For FreeBSD and NetBSD, use st_birthtime. For OpenBSD,
-    // use __st_birthtime.
-
     return true;
 }
 
@@ -990,7 +1087,10 @@ void FileProtocol::special(const QByteArray &data)
 
 static QStringList fallbackSystemPath()
 {
-    return QStringList() << QStringLiteral("/sbin") << QStringLiteral("/bin");
+    return QStringList{
+        QStringLiteral("/sbin"),
+        QStringLiteral("/bin"),
+    };
 }
 
 void FileProtocol::mount(bool _ro, const char *_fstype, const QString &_dev, const QString &_point)
@@ -1032,12 +1132,10 @@ void FileProtocol::mount(bool _ro, const char *_fstype, const QString &_dev, con
     QByteArray dev;
     if (_dev.startsWith(QLatin1String("LABEL="))) { // turn LABEL=foo into -L foo (#71430)
         QString labelName = _dev.mid(6);
-        dev = "-L ";
-        dev += QFile::encodeName(KShell::quoteArg(labelName));     // is it correct to assume same encoding as filesystem?
+        dev = "-L " + QFile::encodeName(KShell::quoteArg(labelName));     // is it correct to assume same encoding as filesystem?
     } else if (_dev.startsWith(QLatin1String("UUID="))) { // and UUID=bar into -U bar
         QString uuidName = _dev.mid(5);
-        dev = "-U ";
-        dev += QFile::encodeName(KShell::quoteArg(uuidName));
+        dev = "-U " + QFile::encodeName(KShell::quoteArg(uuidName));
     } else {
         dev = QFile::encodeName(KShell::quoteArg(_dev));    // get those ready to be given to a shell
     }
@@ -1150,7 +1248,7 @@ void FileProtocol::unmount(const QString &_point)
         // qDebug() << "VOLMGT: looking for "
                 << _point.toLocal8Bit();
 
-        if ((mnttab = QT_FOPEN(MNTTAB, "r")) == NULL) {
+        if ((mnttab = QT_FOPEN(MNTTAB, "r")) == nullptr) {
             err = QLatin1String("could not open mnttab");
             // qDebug() << "VOLMGT: " << err;
             error(KIO::ERR_CANNOT_UNMOUNT, err);
@@ -1163,9 +1261,9 @@ void FileProtocol::unmount(const QString &_point)
          *  media_findname() won't work in this case), we have to
          *  look ourselves...
          */
-        devname = NULL;
+        devname = nullptr;
         rewind(mnttab);
-        while (getmntent(mnttab, &mnt) == 0) {
+        while (getmntent(mnttab, &mnt) == nullptr) {
             if (strcmp(_point.toLocal8Bit(), mnt.mnt_mountp) == 0) {
                 devname = mnt.mnt_special;
                 break;
@@ -1173,7 +1271,7 @@ void FileProtocol::unmount(const QString &_point)
         }
         fclose(mnttab);
 
-        if (devname == NULL) {
+        if (devname == nullptr) {
             err = QLatin1String("not in mnttab");
             // qDebug() << "VOLMGT: "
                     << QFile::encodeName(_point).data()
@@ -1293,9 +1391,7 @@ bool FileProtocol::pumount(const QString &point)
         return false;
     }
 
-    QByteArray buffer = QFile::encodeName(pumountProg);
-    buffer += ' ';
-    buffer += QFile::encodeName(KShell::quoteArg(dev));
+    const QByteArray buffer = QFile::encodeName(pumountProg) + ' ' + QFile::encodeName(KShell::quoteArg(dev));
 
     int res = system(buffer.data());
 
@@ -1359,18 +1455,18 @@ static void appendACLAtoms(const QByteArray &path, UDSEntry &entry, mode_t type)
     }
     if (acl || defaultAcl) {
         // qDebug() << path.constData() << "has extended ACL entries";
-        entry.insert(KIO::UDSEntry::UDS_EXTENDED_ACL, 1);
+        entry.fastInsert(KIO::UDSEntry::UDS_EXTENDED_ACL, 1);
 
         if (acl) {
             const QString str = aclToText(acl);
-            entry.insert(KIO::UDSEntry::UDS_ACL_STRING, str);
+            entry.fastInsert(KIO::UDSEntry::UDS_ACL_STRING, str);
             // qDebug() << path.constData() << "ACL:" << str;
             acl_free(acl);
         }
 
         if (defaultAcl) {
             const QString str = aclToText(defaultAcl);
-            entry.insert(KIO::UDSEntry::UDS_DEFAULT_ACL_STRING, str);
+            entry.fastInsert(KIO::UDSEntry::UDS_DEFAULT_ACL_STRING, str);
             // qDebug() << path.constData() << "DEFAULT ACL:" << str;
             acl_free(defaultAcl);
         }
@@ -1405,7 +1501,7 @@ bool FileProtocol::deleteRecursive(const QString &path)
         }
     }
     QDir dir;
-    Q_FOREACH (const QString &itemPath, dirsToDelete) {
+    for (const QString &itemPath : qAsConst(dirsToDelete)) {
         //qDebug() << "QDir::rmdir" << itemPath;
         if (!dir.rmdir(itemPath)) {
             if (auto err = execWithElevatedPrivilege(RMDIR, {itemPath}, errno)) {
