@@ -20,6 +20,7 @@
 
 #include "kacleditwidget.h"
 #include "kacleditwidget_p.h"
+#include "kio_widgets_debug.h"
 
 #if HAVE_POSIX_ACL
 
@@ -56,12 +57,12 @@ static struct {
     const char *pixmapName;
     QPixmap *pixmap;
 } s_itemAttributes[] = {
-    { I18N_NOOP("Owner"), "user-grey", 0 },
-    { I18N_NOOP("Owning Group"), "group-grey", 0 },
-    { I18N_NOOP("Others"), "others-grey", 0 },
-    { I18N_NOOP("Mask"), "mask", 0 },
-    { I18N_NOOP("Named User"), "user", 0 },
-    { I18N_NOOP("Named Group"), "group", 0 },
+    { I18N_NOOP("Owner"), "user-grey", nullptr },
+    { I18N_NOOP("Owning Group"), "group-grey", nullptr },
+    { I18N_NOOP("Others"), "others-grey", nullptr },
+    { I18N_NOOP("Mask"), "mask", nullptr },
+    { I18N_NOOP("Named User"), "user", nullptr },
+    { I18N_NOOP("Named Group"), "group", nullptr },
 };
 
 class KACLEditWidget::KACLEditWidgetPrivate
@@ -84,7 +85,7 @@ KACLEditWidget::KACLEditWidget(QWidget *parent)
     : QWidget(parent), d(new KACLEditWidgetPrivate)
 {
     QHBoxLayout *hbox = new QHBoxLayout(this);
-    hbox->setMargin(0);
+    hbox->setContentsMargins(0, 0, 0, 0);
     d->m_listView = new KACLListView(this);
     hbox->addWidget(d->m_listView);
     connect(d->m_listView->selectionModel(),
@@ -95,16 +96,16 @@ KACLEditWidget::KACLEditWidget(QWidget *parent)
     hbox->addLayout(vbox);
     d->m_AddBtn = new QPushButton(i18n("Add Entry..."), this);
     vbox->addWidget(d->m_AddBtn);
-    d->m_AddBtn->setObjectName(QLatin1String("add_entry_button"));
-    connect(d->m_AddBtn, SIGNAL(clicked()), d->m_listView, SLOT(slotAddEntry()));
+    d->m_AddBtn->setObjectName(QStringLiteral("add_entry_button"));
+    connect(d->m_AddBtn, &QAbstractButton::clicked, d->m_listView, &KACLListView::slotAddEntry);
     d->m_EditBtn = new QPushButton(i18n("Edit Entry..."), this);
     vbox->addWidget(d->m_EditBtn);
-    d->m_EditBtn->setObjectName(QLatin1String("edit_entry_button"));
-    connect(d->m_EditBtn, SIGNAL(clicked()), d->m_listView, SLOT(slotEditEntry()));
+    d->m_EditBtn->setObjectName(QStringLiteral("edit_entry_button"));
+    connect(d->m_EditBtn, &QAbstractButton::clicked, d->m_listView, &KACLListView::slotEditEntry);
     d->m_DelBtn = new QPushButton(i18n("Delete Entry"), this);
     vbox->addWidget(d->m_DelBtn);
-    d->m_DelBtn->setObjectName(QLatin1String("delete_entry_button"));
-    connect(d->m_DelBtn, SIGNAL(clicked()), d->m_listView, SLOT(slotRemoveEntry()));
+    d->m_DelBtn->setObjectName(QStringLiteral("delete_entry_button"));
+    connect(d->m_DelBtn, &QAbstractButton::clicked, d->m_listView, &KACLListView::slotRemoveEntry);
     vbox->addItem(new QSpacerItem(10, 10, QSizePolicy::Fixed, QSizePolicy::Expanding));
     d->_k_slotUpdateButtons();
 }
@@ -147,12 +148,12 @@ KACL KACLEditWidget::getDefaultACL() const
 
 void KACLEditWidget::setACL(const KACL &acl)
 {
-    return d->m_listView->setACL(acl);
+    d->m_listView->setACL(acl);
 }
 
 void KACLEditWidget::setDefaultACL(const KACL &acl)
 {
-    return d->m_listView->setDefaultACL(acl);
+    d->m_listView->setDefaultACL(acl);
 }
 
 void KACLEditWidget::setAllowDefaults(bool value)
@@ -181,28 +182,28 @@ QString KACLListViewItem::key() const
 {
     QString key;
     if (!isDefault) {
-        key = 'A';
+        key = QLatin1Char('A');
     } else {
-        key = 'B';
+        key = QLatin1Char('B');
     }
     switch (type) {
     case KACLListView::User:
-        key += 'A';
+        key += QLatin1Char('A');
         break;
     case KACLListView::Group:
-        key += 'B';
+        key += QLatin1Char('B');
         break;
     case KACLListView::Others:
-        key += 'C';
+        key += QLatin1Char('C');
         break;
     case KACLListView::Mask:
-        key += 'D';
+        key += QLatin1Char('D');
         break;
     case KACLListView::NamedUser:
-        key += 'E' + text(1);
+        key += QLatin1Char('E') + text(1);
         break;
     case KACLListView::NamedGroup:
-        key += 'F' + text(1);
+        key += QLatin1Char('F') + text(1);
         break;
     default:
         key += text(0);
@@ -314,7 +315,7 @@ void KACLListViewItem::repaint()
 
 void KACLListViewItem::calcEffectiveRights()
 {
-    QString strEffective = QString("---");
+    QString strEffective = QStringLiteral("---");
 
     // Do we need to worry about the mask entry? It applies to named users,
     // owning group, and named groups
@@ -422,9 +423,9 @@ EditACLEntryDialog::EditACLEntryDialog(KACLListView *listView, KACLListViewItem
       m_listView(listView), m_item(item), m_users(users), m_groups(groups),
       m_defaultUsers(defaultUsers), m_defaultGroups(defaultGroups),
       m_allowedTypes(allowedTypes), m_allowedDefaultTypes(allowedDefaultTypes),
-      m_defaultCB(0)
+      m_defaultCB(nullptr)
 {
-    setObjectName("edit_entry_dialog");
+    setObjectName(QStringLiteral("edit_entry_dialog"));
     setModal(true);
     setWindowTitle(i18n("Edit ACL Entry"));
 
@@ -437,49 +438,49 @@ EditACLEntryDialog::EditACLEntryDialog(KACLListView *listView, KACLListViewItem
 
     if (allowDefaults) {
         m_defaultCB = new QCheckBox(i18n("Default for new files in this folder"), this);
-        m_defaultCB->setObjectName(QLatin1String("defaultCB"));
+        m_defaultCB->setObjectName(QStringLiteral("defaultCB"));
         mainLayout->addWidget(m_defaultCB);
-        connect(m_defaultCB, SIGNAL(toggled(bool)),
-                this, SLOT(slotUpdateAllowedUsersAndGroups()));
-        connect(m_defaultCB, SIGNAL(toggled(bool)),
-                this, SLOT(slotUpdateAllowedTypes()));
+        connect(m_defaultCB, &QAbstractButton::toggled,
+                this, &EditACLEntryDialog::slotUpdateAllowedUsersAndGroups);
+        connect(m_defaultCB, &QAbstractButton::toggled,
+                this, &EditACLEntryDialog::slotUpdateAllowedTypes);
     }
 
     QRadioButton *ownerType = new QRadioButton(i18n("Owner"), gb);
-    ownerType->setObjectName(QLatin1String("ownerType"));
+    ownerType->setObjectName(QStringLiteral("ownerType"));
     gbLayout->addWidget(ownerType);
     m_buttonGroup->addButton(ownerType);
     m_buttonIds.insert(ownerType, KACLListView::User);
     QRadioButton *owningGroupType = new QRadioButton(i18n("Owning Group"), gb);
-    owningGroupType->setObjectName(QLatin1String("owningGroupType"));
+    owningGroupType->setObjectName(QStringLiteral("owningGroupType"));
     gbLayout->addWidget(owningGroupType);
     m_buttonGroup->addButton(owningGroupType);
     m_buttonIds.insert(owningGroupType, KACLListView::Group);
     QRadioButton *othersType = new QRadioButton(i18n("Others"), gb);
-    othersType->setObjectName(QLatin1String("othersType"));
+    othersType->setObjectName(QStringLiteral("othersType"));
     gbLayout->addWidget(othersType);
     m_buttonGroup->addButton(othersType);
     m_buttonIds.insert(othersType, KACLListView::Others);
     QRadioButton *maskType = new QRadioButton(i18n("Mask"), gb);
-    maskType->setObjectName(QLatin1String("maskType"));
+    maskType->setObjectName(QStringLiteral("maskType"));
     gbLayout->addWidget(maskType);
     m_buttonGroup->addButton(maskType);
     m_buttonIds.insert(maskType, KACLListView::Mask);
     QRadioButton *namedUserType = new QRadioButton(i18n("Named user"), gb);
-    namedUserType->setObjectName(QLatin1String("namesUserType"));
+    namedUserType->setObjectName(QStringLiteral("namesUserType"));
     gbLayout->addWidget(namedUserType);
     m_buttonGroup->addButton(namedUserType);
     m_buttonIds.insert(namedUserType, KACLListView::NamedUser);
     QRadioButton *namedGroupType = new QRadioButton(i18n("Named group"), gb);
-    namedGroupType->setObjectName(QLatin1String("namedGroupType"));
+    namedGroupType->setObjectName(QStringLiteral("namedGroupType"));
     gbLayout->addWidget(namedGroupType);
     m_buttonGroup->addButton(namedGroupType);
     m_buttonIds.insert(namedGroupType, KACLListView::NamedGroup);
 
     mainLayout->addWidget(gb);
 
-    connect(m_buttonGroup, SIGNAL(buttonClicked(QAbstractButton*)),
-            this, SLOT(slotSelectionChanged(QAbstractButton*)));
+    connect(m_buttonGroup, QOverload<QAbstractButton*>::of(&QButtonGroup::buttonClicked),
+            this, &EditACLEntryDialog::slotSelectionChanged);
 
     m_widgetStack = new QStackedWidget(this);
     mainLayout->addWidget(m_widgetStack);
@@ -493,7 +494,7 @@ EditACLEntryDialog::EditACLEntryDialog(KACLListView *listView, KACLListViewItem
     QLabel *usersLabel = new QLabel(i18n("User: "), usersBox);
     m_usersCombo = new KComboBox(usersBox);
     m_usersCombo->setEditable(false);
-    m_usersCombo->setObjectName(QLatin1String("users"));
+    m_usersCombo->setObjectName(QStringLiteral("users"));
     usersLabel->setBuddy(m_usersCombo);
 
     usersLayout->addWidget(usersLabel);
@@ -508,7 +509,7 @@ EditACLEntryDialog::EditACLEntryDialog(KACLListView *listView, KACLListViewItem
     QLabel *groupsLabel = new QLabel(i18n("Group: "), groupsBox);
     m_groupsCombo = new KComboBox(groupsBox);
     m_groupsCombo->setEditable(false);
-    m_groupsCombo->setObjectName(QLatin1String("groups"));
+    m_groupsCombo->setObjectName(QStringLiteral("groups"));
     groupsLabel->setBuddy(m_groupsCombo);
 
     groupsLayout->addWidget(groupsLabel);
@@ -537,8 +538,8 @@ EditACLEntryDialog::EditACLEntryDialog(KACLListView *listView, KACLListViewItem
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(this);
     buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    connect(buttonBox, SIGNAL(accepted()), this, SLOT(slotOk()));
-    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+    connect(buttonBox, &QDialogButtonBox::accepted, this, &EditACLEntryDialog::slotOk);
+    connect(buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
     mainLayout->addWidget(buttonBox);
 
     adjustSize();
@@ -589,7 +590,7 @@ void EditACLEntryDialog::slotOk()
 {
     KACLListView::EntryType type = static_cast<KACLListView::EntryType>(m_buttonIds[m_buttonGroup->checkedButton()]);
 
-    qWarning() << "Type 2: " << type;
+    qCWarning(KIO_WIDGETS) << "Type 2: " << type;
 
     QString qualifier;
     if (type == KACLListView::NamedUser) {
@@ -641,13 +642,14 @@ KACLListView::KACLListView(QWidget *parent)
 {
     // Add the columns
     setColumnCount(6);
-    QStringList headers;
-    headers <<  i18n("Type");
-    headers <<  i18n("Name");
-    headers <<  i18nc("read permission", "r");
-    headers <<  i18nc("write permission", "w");
-    headers <<  i18nc("execute permission", "x");
-    headers <<  i18n("Effective");
+    const QStringList headers {
+        i18n("Type"),
+        i18n("Name"),
+        i18nc("read permission", "r"),
+        i18nc("write permission", "w"),
+        i18nc("execute permission", "x"),
+        i18n("Effective"),
+    };
     setHeaderLabels(headers);
 
     setSortingEnabled(false);
@@ -657,30 +659,33 @@ KACLListView::KACLListView(QWidget *parent)
 
     // Load the avatars
     for (int i = 0; i < LAST_IDX; ++i) {
-        s_itemAttributes[i].pixmap = new QPixmap(QString::fromLatin1(":/images/%1").arg(s_itemAttributes[i].pixmapName));
+        s_itemAttributes[i].pixmap = new QPixmap(QLatin1String(":/images/") + QLatin1String(s_itemAttributes[i].pixmapName));
     }
-    m_yesPixmap = new QPixmap(":/images/yes.png");
-    m_yesPartialPixmap = new QPixmap(":/images/yespartial.png");
+    m_yesPixmap = new QPixmap(QStringLiteral(":/images/yes.png"));
+    m_yesPartialPixmap = new QPixmap(QStringLiteral(":/images/yespartial.png"));
 
     // fill the lists of all legal users and groups
-    struct passwd *user = 0;
+    struct passwd *user = nullptr;
     setpwent();
-    while ((user = getpwent()) != 0) {
+    while ((user = getpwent()) != nullptr) {
         m_allUsers << QString::fromLatin1(user->pw_name);
     }
     endpwent();
 
-    struct group *gr = 0;
+    struct group *gr = nullptr;
     setgrent();
-    while ((gr = getgrent()) != 0) {
+    while ((gr = getgrent()) != nullptr) {
         m_allGroups << QString::fromLatin1(gr->gr_name);
     }
     endgrent();
     m_allUsers.sort();
     m_allGroups.sort();
 
-    connect(this, SIGNAL(itemClicked(QTreeWidgetItem*,int)),
-            this, SLOT(slotItemClicked(QTreeWidgetItem*,int)));
+    connect(this, &QTreeWidget::itemClicked,
+            this, &KACLListView::slotItemClicked);
+
+    connect(this, &KACLListView::itemDoubleClicked,
+            this, &KACLListView::slotItemDoubleClicked);
 }
 
 KACLListView::~KACLListView()
@@ -938,11 +943,31 @@ void KACLListView::slotItemClicked(QTreeWidgetItem *pItem,  int col)
      */
 }
 
+void KACLListView::slotItemDoubleClicked(QTreeWidgetItem *item, int column)
+{
+    if (!item) {
+        return;
+    }
+
+    // avoid conflict with clicking to toggle permission
+    if (column >= 2 && column <= 4) {
+        return;
+    }
+
+    KACLListViewItem *aclListItem = static_cast<KACLListViewItem *>(item);
+    if (!aclListItem->isAllowedToChangeType()) {
+        return;
+    }
+
+    setCurrentItem(item);
+    slotEditEntry();
+}
+
 void KACLListView::calculateEffectiveRights()
 {
     QTreeWidgetItemIterator it(this);
     KACLListViewItem *pItem;
-    while ((pItem = dynamic_cast<KACLListViewItem *>(*it)) != 0) {
+    while ((pItem = dynamic_cast<KACLListViewItem *>(*it)) != nullptr) {
         ++it;
         pItem->calcEffectiveRights();
     }
@@ -999,7 +1024,7 @@ const KACLListViewItem *KACLListView::findItemByType(EntryType type, bool defaul
             return item;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 unsigned short KACLListView::calculateMaskValue(bool defaults) const
@@ -1022,7 +1047,7 @@ void KACLListView::slotAddEntry()
     if (!hasDefaultEntries()) {
         allowedDefaultTypes |= User | Group;
     }
-    EditACLEntryDialog dlg(this, 0,
+    EditACLEntryDialog dlg(this, nullptr,
                            allowedUsers(false), allowedGroups(false),
                            allowedUsers(true), allowedGroups(true),
                            allowedTypes, allowedDefaultTypes, m_allowDefaults);
