@@ -188,6 +188,7 @@ public:
     }
 
     void _k_slotNewItems(const QUrl &directoryUrl, const KFileItemList &);
+    void _k_slotCompleted(const QUrl &directoryUrl);
     void _k_slotDeleteItems(const KFileItemList &);
     void _k_slotRefreshItems(const QList<QPair<KFileItem, KFileItem> > &);
     void _k_slotClear();
@@ -259,7 +260,7 @@ void KDirModelPrivate::removeFromNodeHash(KDirModelNode *node, const QUrl &url)
     if (node->item().isDir()) {
         QList<QUrl> urls;
         static_cast<KDirModelDirNode *>(node)->collectAllChildUrls(urls);
-        Q_FOREACH (const QUrl &u, urls) {
+        for (const QUrl &u : qAsConst(urls)) {
             m_nodeHash.remove(u);
         }
     }
@@ -406,6 +407,8 @@ void KDirModel::setDirLister(KDirLister *dirLister)
     d->m_dirLister->setParent(this);
     connect(d->m_dirLister, &KCoreDirLister::itemsAdded, this,
         [this](const QUrl &dirUrl, const KFileItemList &items){d->_k_slotNewItems(dirUrl, items);} );
+    connect(d->m_dirLister, static_cast<void (KCoreDirLister::*)(const QUrl&)>(&KCoreDirLister::completed), this,
+        [this](const QUrl &dirUrl){d->_k_slotCompleted(dirUrl);} );
     connect(d->m_dirLister, &KCoreDirLister::itemsDeleted, this,
         [this](const KFileItemList &items){d->_k_slotDeleteItems(items);} );
     connect(d->m_dirLister, &KCoreDirLister::refreshItems, this,
@@ -432,7 +435,7 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
         qCWarning(KIO_WIDGETS) << "Items emitted in directory" << directoryUrl
                    << "but that directory isn't in KDirModel!"
                    << "Root directory:" << urlForNode(m_rootNode);
-        Q_FOREACH (const KFileItem &item, items) {
+        for (const KFileItem &item : items) {
             qDebug() << "Item:" << item.url();
         }
 #ifndef NDEBUG
@@ -500,8 +503,6 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
         }
     }
 
-    m_urlsBeingFetched.remove(dirNode);
-
     q->endInsertRows();
 
     // Emit expand signal after rowsInserted signal has been emitted,
@@ -511,6 +512,14 @@ void KDirModelPrivate::_k_slotNewItems(const QUrl &directoryUrl, const KFileItem
     }
 }
 
+void KDirModelPrivate::_k_slotCompleted(const QUrl &directoryUrl)
+{
+    KDirModelNode *result = nodeForUrl(directoryUrl); // O(depth)
+    Q_ASSERT(isDir(result));
+    KDirModelDirNode *dirNode = static_cast<KDirModelDirNode *>(result);
+    m_urlsBeingFetched.remove(dirNode);
+}
+
 void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
 {
     //qDebug() << items.count();
@@ -547,7 +556,7 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList &items)
     // Let's use a bit array where each bit represents a given child node.
     const int childCount = dirNode->m_childNodes.count();
     QBitArray rowNumbers(childCount, false);
-    Q_FOREACH (const KFileItem &item, items) {
+    for (const KFileItem &item : items) {
         if (!node) { // don't lookup the first item twice
             url = item.url();
             node = nodeForUrl(url);
@@ -1033,7 +1042,7 @@ QMimeData *KDirModel::mimeData(const QModelIndexList &indexes) const
     urls.reserve(indexes.size());
     mostLocalUrls.reserve(indexes.size());
     bool canUseMostLocalUrls = true;
-    foreach (const QModelIndex &index, indexes) {
+    for (const QModelIndex &index : indexes) {
         const KFileItem &item = d->nodeForIndex(index)->item();
         urls << item.url();
         bool isLocal;
